#include "Vector.h"
#include "CSRMatrix.h"
// конструктор по умолчанию
CSRMatrix::CSRMatrix()
    : elementsSize(0)
    , columnsSize(0)
    , rowPointersSize(0)
{}

// конструктор матрицы
CSRMatrix::CSRMatrix(
    size_t  elementsSize,
    size_t  columnsSize,
    size_t  rowPointersSize)
    : elementsSize(elementsSize)
    , columnsSize(columnsSize)
    , rowPointersSize(rowPointersSize)
{
    elements = new double[elementsSize];
    columns = new int[columnsSize];
    rowPointers = new int[rowPointersSize];
}

// конструктор копирования
CSRMatrix::CSRMatrix(const CSRMatrix& other) 
    : elementsSize(other.elementsSize)
    , columnsSize(other.columnsSize)
    , rowPointersSize(other.rowPointersSize)
{
    elements = new double[elementsSize];
    columns = new int[columnsSize];
    rowPointers = new int[rowPointersSize];
    for (int i = 0; i < elementsSize; ++i)
    {
        this->elements[i] = other.elements[i];
    }
    for (int i = 0; i < columnsSize; ++i)
    {
        this->columns[i] = other.columns[i];
    }
    for (int i = 0; i < rowPointersSize; ++i)
    {
        this->rowPointers[i] = other.rowPointers[i];
    }
}

// деструктор
CSRMatrix::~CSRMatrix() 
{
    delete[] elements;
    delete[] columns;
    delete[] rowPointers;
}

// метод, возвращающий количество строк в матрице
size_t CSRMatrix::GetRows() const 
{
    return rowPointersSize;
}

// метод, возвращающий количество столбцов в матрице
size_t CSRMatrix::GetColumns() const 
{
    return columnsSize;
}

// метод, вычисляющий количество ненулевых элементов
size_t CSRMatrix::NonZeroElements() const
{
    return elementsSize;
}

//перегрузка оператора присваивания = для матриц
CSRMatrix& CSRMatrix::operator = (const CSRMatrix& other)
{
    if (this != &other)
    {
        if (this->elementsSize != other.elementsSize)// Выделять память заново нужно только, если размеры не совпадают
        {
            this->elementsSize = other.elementsSize;
            this->elements = new double[other.elementsSize];
        }

        for (int i = 0; i < elementsSize; ++i)
        {
            this->elements[i] = other.elements[i];
        }
        
        if (this->columns != nullptr)
        {
            delete[] this->columns;
        }
        this->columnsSize = other.columnsSize;
        this->columns = new int[other.columnsSize];
        for (int i = 0; i < columnsSize; ++i)
        {
            this->columns[i] = other.columns[i];
        }

        if (this->rowPointers != nullptr)
        {
            delete[] this->rowPointers;
        }
        this->rowPointersSize = other.rowPointersSize;
        this->rowPointers = new int[other.rowPointersSize];
        for (int i = 0; i < rowPointersSize; ++i)
        {
            this->rowPointers[i] = other.rowPointers[i];
        }
    }
    return *this;
}

// перегрузка оператора умножения матрицы(массив ненулевых чисел) на число
CSRMatrix operator* (const CSRMatrix& matrix, double value) 
{
    CSRMatrix temp(matrix);
    for (size_t i = 0; i < temp.elementsSize; ++i) 
    {
        temp.elements[i] *= value;
    }
    return temp;
}

// перегрузка оператора умножения числа на матрицу (массив ненулевых чисел)
CSRMatrix operator *(double value, const CSRMatrix& matrix)
{
    CSRMatrix temp(matrix);
    for (int i = 0; i < temp.elementsSize; ++i)
    {
        temp.elements[i] *= value;
    }
    return temp;
}

// перегрузка оператора умножения матрицы на вектор
Vector CSRMatrix::operator*(const Vector& vector) const 
{
    int size = vector.getsize();
    if (rowPointersSize - 1 != size) 
    {
        throw InCompatibleDimException();
    }
    Vector temp(size);
    for (int row = 0; row < size; ++row) {
        for (int i = rowPointers[row]; i < rowPointers[row + 1]; ++i) 
        {
            temp[row] += elements[i] * elements[columns[i]];
        }
    }
    return temp;

}

// перегрузка оператора умножения вектора на матрицу
Vector operator*(const Vector& vector, const CSRMatrix& matrix)
{
    int matrixSize = matrix.GetRows()-1;
    int vectorSize = vector.getsize();
    Vector temp(matrixSize);
    if (matrixSize != vectorSize) 
    {
        throw InCompatibleDimException();
    }
    for (int row = 0; row < matrixSize; ++row) 
    {
        for (int i = matrix.rowPointers[row]; i < matrix.rowPointers[row + 1]; ++i) 
        {
            temp[matrix.columns[i]] += matrix.elements[i] * vector[row];
        }
    }
    return temp;
} 

// оператор ввода в поток для матриц
std::istream& operator >> (std::istream& in, CSRMatrix& matrix)
{
    std::cout << "Введите массив ненулевых элементов" << "\n";
    for (int i = 0; i <matrix.elementsSize; ++i)
    {
        in >> matrix.elements[i];
    }
    std::cout << "Введите столбцов, в которых находятся эти ненулевые элементы" << "\n";
    for (int i = 0; i < matrix.columnsSize; ++i)
    {
        in >> matrix.columns[i];
    }
    std::cout << "Введите массив индексации строк" << "\n";
    for (int i = 0; i < matrix.rowPointersSize; ++i)
    {
        in >> matrix.rowPointers[i];
    }
    return in;
}

// оператор вывода из потока для матриц
std::ostream& operator <<(std::ostream& out, const CSRMatrix& matrix)
{
    int index = 0;
    for (int i = 0; i < matrix.columns[matrix.columnsSize - 1] + 1; ++i) // i < matrix.elementsSize -1
    {
        std::cout << " [";
        for (int j = 0; j < matrix.columns[matrix.columnsSize - 1] + 1; ++j) // j < matrix.elemenstSize -1
        {
            if (j == matrix.columns[index])
            {
                std::cout << std::setw(5) << matrix.elements[index];
                ++index;
            }
            else
                std::cout << std::setw(5) << 0;
            
        }
        if (matrix.elementsSize - 1 == matrix.columns[index])
            {
                std::cout << std::setw(5) << matrix.elements[index] << std:: setw(5) <<"]";
                ++index;
            }
            else
                std::cout << std::setw(5) <<"]";
        std::cout << "\n\n";
    }

    for (int i = 0; i < matrix.elementsSize; ++i)
    {
        out << matrix.elements[i] << " ";
    }
    std::cout << " Массив ненулевых элементов\n";
    
    for (int i = 0; i < matrix.columnsSize; ++i) 
    {
        out << matrix.columns[i] << " ";
    }
    std::cout << " Массив столбцов, в которых находятся эти ненулевые элементы\n";
    
    for (int i = 0; i < matrix.rowPointersSize; ++i) 
    {
        out << matrix.rowPointers[i] << " ";
    }
    std::cout << " Массив индексации строк\n";
    return out;
}





